#include "cutTex.h"
#include <stdlib.h>

textureCutter::textureCutter(char* aFileName, int aDegSize):
   degSize(aDegSize)
{
   fileName = new char[strlen(aFileName)+1];
   memcpy(fileName, aFileName, strlen(aFileName));
   fileName[strlen(aFileName)+1] = '\0';
   data = NULL;
   boundariesY[0] = -1;
}

textureCutter::textureCutter(char* aFileName, int aDegSize,
			     int aBoundariesY[]):
   degSize(aDegSize)
{
   fileName = new char[strlen(aFileName)+1];
   memcpy(fileName, aFileName, strlen(aFileName));
   fileName[strlen(aFileName)+1] = '\0';
   data = NULL;
   for(int i=0; aBoundariesY[i] >= 0; i++)
   {
      boundariesY[i] = aBoundariesY[i];
      cerr << "b[" << i << "]=" << boundariesY[i] << endl;
   }
   boundariesY[i] = -1;
}

textureCutter::~textureCutter()
{
   delete[] fileName;
   if(data!=NULL)
      delete[] data;
}

void
textureCutter::run()
{
   if(parseFileName() && read())
   {
      calculateBoundaries();
      cutImage();
   }
}

int
textureCutter::parseFileName()
{
   int dLon, mLon, dLat, mLat;
   int dsLon, msLon, dsLat, msLat;

   sscanf(fileName, "%d:%d,%d:%d,%d:%d,%d:%d.ppm",
	  &dLon, &mLon, &dLat, &mLat,
	  &dsLon, &msLon, &dsLat, &msLat);

   ulLon = dLon*60 + mLon;
   ulLat = dLat*60 + mLat;
   szLon = dsLon*60 + msLon;
   szLat = dsLat*60 + msLat;
   
   return 1;
}

int
textureCutter::read()
{
   char charJunk;

   ifstream image(fileName);
   if(!image.good())
   {
      cerr << "Cannot open image file for reading!\n\n";
      return 0;
   }

   //P5\n
   image.get(charJunk);
   image.get(charJunk);
   image.get(charJunk);

   int colorResolution;
   image >> totalWidth >> totalHeight >> colorResolution;
   if(colorResolution!=255)
   {
      cerr << "Something wrong when reading color resolution!\n\n";
      return 0;
   }

   if(!allocateMemory())
   {
      cerr << "Cannot allocate memory!\n\n";
      return 0;
   }

   image.get(charJunk);
   image.read(data, totalHeight*totalWidth*3);

   image.close();
   return 1;
}

void
textureCutter::calculateBoundaries()
{
   int chunksOnX = szLon / degSize;
   int chunksOnY = szLat / degSize;
   int chunkHeight = totalHeight / chunksOnY;
   int chunkWidth = totalWidth / chunksOnX;

   for(int col = 0; col<=chunksOnX; col++)
      boundariesX[col] = col * chunkWidth;

   if(boundariesY[0]>=0)
      return;
   
   for(int row = 0; row<=chunksOnY; row++)
      boundariesY[row] = row * chunkHeight;
}

int
textureCutter::cutImage()
{
   int chunksOnX = szLon / degSize;
   int chunksOnY = szLat / degSize;

   for(int row = 0; row<chunksOnY; row++)
      for(int col = 0; col<chunksOnX; col++)
	 if(!saveChunk(row, col))
	 {
	    cerr << "Problems saving a chunk!\n\n";
	    return(0);
	 }
   
   return 1;
}

int
textureCutter::allocateMemory()
{
   return ((data=new char[totalHeight*totalWidth*3])==NULL)? 0: 1;
}

int
textureCutter::saveChunk(int row, int col)
{
   char chunkBaseName[256], chunkName[256];

   int north = ulLat - row * degSize;
   int west = ulLon - col * degSize;
   
   sprintf(chunkBaseName, "%d:%d:0,%d:%d:0.ppm",
	   west/60, west%60, north/60, north%60);

   int suffix = 0, found = 0;

   do
   {
      if(suffix)
	 sprintf(chunkName, "split/%s%d", chunkBaseName, suffix);
      else
	 sprintf(chunkName, "split/%s", chunkBaseName);
      ifstream testExistance(chunkName);
      found = testExistance.good();
      suffix++;
   } while(found);
   
   ofstream chunk(chunkName);

   chunk << "P6" << endl
	 << boundariesX[col+1] - boundariesX[col] << ' '
	 << boundariesY[row+1] - boundariesY[row] << endl
	 << 255 << endl;
   
   for(int y = boundariesY[row]; y<boundariesY[row+1]; y++)
      for(int x = boundariesX[col]; x<boundariesX[col+1]; x++)
      {
	 chunk.put(data[(y*totalWidth+x)*3+0]);
	 chunk.put(data[(y*totalWidth+x)*3+1]);
	 chunk.put(data[(y*totalWidth+x)*3+2]);
      }

   chunk.close();
   return 1;
}



/////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[])
{
   if(argc<3)
   {
      cerr << "Usage:\n"
	   << argv[0] << " ppmfile chunksize {thresholds}\n"
	   << "chunksize in minutes\n"
	   << "If thresholds are specified, they should be numbers of rows\n"
	   << "where each chunk BEGINS, starting from 0 (ZERO)\n\n";
      exit(-1);
   }

   if(argc==3)
   {
      textureCutter tC(argv[1], atoi(argv[2]));
      tC.run();
   }
   else
   {
      int boundariesY[20];
      for(int i = 3; i<argc+1; i++)
	 boundariesY[i-3] = atoi(argv[i]);
      boundariesY[i-4] = -1;
      textureCutter tC(argv[1], atoi(argv[2]), boundariesY);
      tC.run();
   }
};

